@@ -25,91 +25,34 @@ import sys
 import time
 
 import cryptography
+from eventlet import greenthread
 import glanceclient
-from glanceclient.common import http
 import glanceclient.exc
-from oslo_config import cfg
+from glanceclient.v2 import schemas
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from oslo_service import sslutils
 from oslo_utils import excutils
-from oslo_utils import netutils
 from oslo_utils import timeutils
 import six
 from six.moves import range
 import six.moves.urllib.parse as urlparse
 
+import nova.conf
 from nova import exception
 from nova.i18n import _LE, _LI, _LW
 import nova.image.download as image_xfers
 from nova import objects
+from nova.objects import fields
 from nova import signature_utils
 
-
-glance_opts = [
-    cfg.StrOpt('host',
-               default='$my_ip',
-               # TODO(sdague): remove in N
-               deprecated_for_removal=True,
-               help='DEPRECATED: Glance server hostname or IP address. '
-                    'Use the "api_servers" option instead.'),
-    cfg.IntOpt('port',
-               default=9292,
-               min=1,
-               max=65535,
-               # TODO(sdague): remove in N
-               deprecated_for_removal=True,
-               help='DEPRECATED: Glance server port. Use the "api_servers" '
-                    'option instead.'),
-    cfg.StrOpt('protocol',
-                default='http',
-                choices=('http', 'https'),
-                # TODO(sdague): remove in N
-                deprecated_for_removal=True,
-                help='DEPRECATED: Protocol to use when connecting to glance. '
-                     'Set to https for SSL. Use the "api_servers" option '
-                     'instead.'),
-    cfg.ListOpt('api_servers',
-                help='''
-A list of the glance api servers endpoints available to nova. These
-should be fully qualified urls of the form
-"scheme://hostname:port[/path]" (i.e. "http://10.0.1.0:9292" or
-"https://my.glance.server/image")'''),
-    cfg.BoolOpt('api_insecure',
-                default=False,
-                help='Allow to perform insecure SSL (https) requests to '
-                     'glance'),
-    cfg.IntOpt('num_retries',
-               default=0,
-               help='Number of retries when uploading / downloading an image '
-                    'to / from glance.'),
-    cfg.ListOpt('allowed_direct_url_schemes',
-                default=[],
-                help='A list of url scheme that can be downloaded directly '
-                     'via the direct_url.  Currently supported schemes: '
-                     '[file].'),
-    cfg.BoolOpt('verify_glance_signatures',
-                default=False,
-                help='Require Nova to perform signature verification on '
-                     'each image downloaded from Glance.'),
-    ]
-
 LOG = logging.getLogger(__name__)
-CONF = cfg.CONF
-CONF.register_opts(glance_opts, 'glance')
-CONF.import_opt('auth_strategy', 'nova.api.auth')
-CONF.import_opt('my_ip', 'nova.netconf')
-
-supported_glance_versions = (1, 2)
+CONF = nova.conf.CONF
 
 
 def generate_glance_url():
-    """Generate the URL to glance."""
-    glance_host = CONF.glance.host
-    if netutils.is_valid_ipv6(glance_host):
-        glance_host = '[%s]' % glance_host
-    return "%s://%s:%d" % (CONF.glance.protocol, glance_host,
-                           CONF.glance.port)
+    """Return a random glance url from the api servers we know about."""
+    return next(get_api_servers())
 
 
 def generate_image_url(image_ref):
@@ -142,42 +85,23 @@ def generate_identity_headers(context, status='Confirmed'):
 
 
 def _glanceclient_from_endpoint(context, endpoint, version=1):
-        """Instantiate a new glanceclient.Client object."""
-        params = {}
-        # NOTE(sdague): even if we aren't using keystone, it doesn't
-        # hurt to send these headers.
-        params['identity_headers'] = generate_identity_headers(context)
-        if endpoint.startswith('https://'):
-            # https specific params
-            params['insecure'] = CONF.glance.api_insecure
-            params['ssl_compression'] = False
-            sslutils.is_enabled(CONF)
-            if CONF.ssl.cert_file:
-                params['cert_file'] = CONF.ssl.cert_file
-            if CONF.ssl.key_file:
-                params['key_file'] = CONF.ssl.key_file
-            if CONF.ssl.ca_file:
-                params['cacert'] = CONF.ssl.ca_file
-        return glanceclient.Client(str(version), endpoint, **params)
-
-
-def _determine_curr_major_version(endpoint):
-    """Determines the current major version of the glance API in use
-
-    :returns Integer version number or None if unable to determine version
-    """
-    http_client = http.HTTPClient(endpoint)
-    try:
-        response, content = http_client.get('/versions')
-        for version in content['versions']:
-            if version['status'] == 'CURRENT':
-                res = version['id']
-                # The 'id' value looks like "v2.2",
-                # so grab the major version number which is 2 in this case
-                res = int(res[1:res.find(".")])
-                return res if res in supported_glance_versions else None
-    except Exception:
-        LOG.error(_LE("Unable to determine the glance API version"))
+    """Instantiate a new glanceclient.Client object."""
+    params = {}
+    # NOTE(sdague): even if we aren't using keystone, it doesn't
+    # hurt to send these headers.
+    params['identity_headers'] = generate_identity_headers(context)
+    if endpoint.startswith('https://'):
+        # https specific params
+        params['insecure'] = CONF.glance.api_insecure
+        params['ssl_compression'] = False
+        sslutils.is_enabled(CONF)
+        if CONF.ssl.cert_file:
+            params['cert_file'] = CONF.ssl.cert_file
+        if CONF.ssl.key_file:
+            params['key_file'] = CONF.ssl.key_file
+        if CONF.ssl.ca_file:
+            params['cacert'] = CONF.ssl.ca_file
+    return glanceclient.Client(str(version), endpoint, **params)
 
 
 def get_api_servers():
@@ -187,14 +111,11 @@ def get_api_servers():
     """
     api_servers = []
 
-    configured_servers = ([generate_glance_url()]
-                          if CONF.glance.api_servers is None
-                          else CONF.glance.api_servers)
-    for api_server in configured_servers:
+    for api_server in CONF.glance.api_servers:
         if '//' not in api_server:
             api_server = 'http://' + api_server
-            # NOTE(sdague): remove in N.
-            LOG.warn(
+            # NOTE(sdague): remove in O.
+            LOG.warning(
                 _LW("No protocol specified in for api_server '%s', "
                     "please update [glance] api_servers with fully "
                     "qualified url including scheme (http / https)"),
@@ -207,7 +128,11 @@ def get_api_servers():
 class GlanceClientWrapper(object):
     """Glance client wrapper class that implements retries."""
 
-    def __init__(self, context=None, endpoint=None, version=1):
+    def __init__(self, context=None, endpoint=None):
+        if CONF.glance.use_glance_v1:
+            version = 1
+        else:
+            version = 2
         if endpoint is not None:
             self.client = self._create_static_client(context,
                                                      endpoint,
@@ -247,7 +172,9 @@ class GlanceClientWrapper(object):
             client = self.client or self._create_onetime_client(context,
                                                                 version)
             try:
-                result = getattr(client.images, method)(*args, **kwargs)
+                controller = getattr(client,
+                                     kwargs.pop('controller', 'images'))
+                result = getattr(controller, method)(*args, **kwargs)
                 if inspect.isgenerator(result):
                     # Convert generator results to a list, so that we can
                     # catch any potential exceptions now and retry the call.
@@ -421,6 +348,7 @@ class GlanceImageService(object):
                 try:
                     for chunk in image_chunks:
                         verifier.update(chunk)
+                        greenthread.sleep(0)
                     verifier.verify()
 
                     LOG.info(_LI('Image signature verification succeeded '
@@ -504,6 +432,351 @@ class GlanceImageService(object):
         return True
 
 
+class GlanceImageServiceV2(object):
+    """Provides storage and retrieval of disk image objects within Glance."""
+
+    def __init__(self, client=None):
+        self._client = client or GlanceClientWrapper()
+        # NOTE(jbresnah) build the table of download handlers at the beginning
+        # so that operators can catch errors at load time rather than whenever
+        # a user attempts to use a module.  Note this cannot be done in glance
+        # space when this python module is loaded because the download module
+        # may require configuration options to be parsed.
+        self._download_handlers = {}
+        download_modules = image_xfers.load_transfer_modules()
+
+        for scheme, mod in six.iteritems(download_modules):
+            if scheme not in CONF.glance.allowed_direct_url_schemes:
+                continue
+
+            try:
+                self._download_handlers[scheme] = mod.get_download_handler()
+            except Exception as ex:
+                LOG.error(_LE('When loading the module %(module_str)s the '
+                              'following error occurred: %(ex)s'),
+                          {'module_str': str(mod), 'ex': ex})
+
+    def show(self, context, image_id, include_locations=False,
+             show_deleted=True):
+        """Returns a dict with image data for the given opaque image id.
+
+        :param context: The context object to pass to image client
+        :param image_id: The UUID of the image
+        :param include_locations: (Optional) include locations in the returned
+                                  dict of information if the image service API
+                                  supports it. If the image service API does
+                                  not support the locations attribute, it will
+                                  still be included in the returned dict, as an
+                                  empty list.
+        :param show_deleted: (Optional) show the image even the status of
+                             image is deleted.
+        """
+        try:
+            image = self._client.call(context, 2, 'get', image_id)
+        except Exception:
+            _reraise_translated_image_exception(image_id)
+
+        if not show_deleted and getattr(image, 'deleted', False):
+            raise exception.ImageNotFound(image_id=image_id)
+
+        if not _is_image_available(context, image):
+            raise exception.ImageNotFound(image_id=image_id)
+
+        image = _translate_from_glance(image,
+                                       include_locations=include_locations)
+        if include_locations:
+            locations = image.get('locations', None) or []
+            du = image.get('direct_url', None)
+            if du:
+                locations.append({'url': du, 'metadata': {}})
+            image['locations'] = locations
+
+        return image
+
+    def _get_transfer_module(self, scheme):
+        try:
+            return self._download_handlers[scheme]
+        except KeyError:
+            return None
+        except Exception:
+            LOG.error(_LE("Failed to instantiate the download handler "
+                          "for %(scheme)s"), {'scheme': scheme})
+        return
+
+    def detail(self, context, **kwargs):
+        """Calls out to Glance for a list of detailed image information."""
+        params = _extract_query_params_v2(kwargs)
+        try:
+            images = self._client.call(context, 2, 'list', **params)
+        except Exception:
+            _reraise_translated_exception()
+
+        _images = []
+        for image in images:
+            if _is_image_available(context, image):
+                _images.append(_translate_from_glance(image))
+
+        return _images
+
+    def download(self, context, image_id, data=None, dst_path=None):
+        """Calls out to Glance for data and writes data."""
+        if CONF.glance.allowed_direct_url_schemes and dst_path is not None:
+            image = self.show(context, image_id, include_locations=True)
+            for entry in image.get('locations', []):
+                loc_url = entry['url']
+                loc_meta = entry['metadata']
+                o = urlparse.urlparse(loc_url)
+                xfer_mod = self._get_transfer_module(o.scheme)
+                if xfer_mod:
+                    try:
+                        xfer_mod.download(context, o, dst_path, loc_meta)
+                        LOG.info(_LI("Successfully transferred "
+                                     "using %s"), o.scheme)
+                        return
+                    except Exception:
+                        LOG.exception(_LE("Download image error"))
+
+        try:
+            image_chunks = self._client.call(context, 2, 'data', image_id)
+        except Exception:
+            _reraise_translated_image_exception(image_id)
+
+        # Retrieve properties for verification of Glance image signature
+        verifier = None
+        if CONF.glance.verify_glance_signatures:
+            image_meta_dict = self.show(context, image_id,
+                                        include_locations=False)
+            image_meta = objects.ImageMeta.from_dict(image_meta_dict)
+            img_signature = image_meta.properties.get('img_signature')
+            img_sig_hash_method = image_meta.properties.get(
+                'img_signature_hash_method'
+            )
+            img_sig_cert_uuid = image_meta.properties.get(
+                'img_signature_certificate_uuid'
+            )
+            img_sig_key_type = image_meta.properties.get(
+                'img_signature_key_type'
+            )
+            try:
+                verifier = signature_utils.get_verifier(context,
+                                                        img_sig_cert_uuid,
+                                                        img_sig_hash_method,
+                                                        img_signature,
+                                                        img_sig_key_type)
+            except exception.SignatureVerificationError:
+                with excutils.save_and_reraise_exception():
+                    LOG.error(_LE('Image signature verification failed '
+                                  'for image: %s'), image_id)
+
+        close_file = False
+        if data is None and dst_path:
+            data = open(dst_path, 'wb')
+            close_file = True
+
+        if data is None:
+
+            # Perform image signature verification
+            if verifier:
+                try:
+                    for chunk in image_chunks:
+                        verifier.update(chunk)
+                    verifier.verify()
+
+                    LOG.info(_LI('Image signature verification succeeded '
+                                 'for image: %s'), image_id)
+
+                except cryptography.exceptions.InvalidSignature:
+                    with excutils.save_and_reraise_exception():
+                        LOG.error(_LE('Image signature verification failed '
+                                      'for image: %s'), image_id)
+            return image_chunks
+        else:
+            try:
+                for chunk in image_chunks:
+                    if verifier:
+                        verifier.update(chunk)
+                    data.write(chunk)
+                if verifier:
+                    verifier.verify()
+                    LOG.info(_LI('Image signature verification succeeded '
+                                 'for image %s'), image_id)
+            except cryptography.exceptions.InvalidSignature:
+                data.truncate(0)
+                with excutils.save_and_reraise_exception():
+                    LOG.error(_LE('Image signature verification failed '
+                                  'for image: %s'), image_id)
+            except Exception as ex:
+                with excutils.save_and_reraise_exception():
+                    LOG.error(_LE("Error writing to %(path)s: %(exception)s"),
+                              {'path': dst_path, 'exception': ex})
+            finally:
+                if close_file:
+                    data.close()
+
+    def create(self, context, image_meta, data=None):
+        """Store the image data and return the new image object."""
+        # Here we workaround the situation when user wants to activate an
+        # empty image right after the creation. In Glance v1 api (and
+        # therefore in Nova) it is enough to set 'size = 0'. v2 api
+        # doesn't allow this hack - we have to send an upload request with
+        # empty data.
+        force_activate = data is None and image_meta.get('size') == 0
+
+        sent_service_image_meta = _translate_to_glance(image_meta)
+
+        try:
+            image = self._create_v2(context, sent_service_image_meta,
+                                    data, force_activate)
+        except glanceclient.exc.HTTPException:
+            _reraise_translated_exception()
+
+        return _translate_from_glance(image)
+
+    def _add_location(self, context, image_id, location):
+        # 'show_multiple_locations' must be enabled in glance api conf file.
+        try:
+            return self._client.call(context, 2, 'add_location', image_id,
+                                     location, {})
+        except glanceclient.exc.HTTPBadRequest:
+            _reraise_translated_exception()
+
+    def _upload_data(self, context, image_id, data):
+        self._client.call(context, 2, 'upload', image_id, data)
+        return self._client.call(context, 2, 'get', image_id)
+
+    def _get_image_create_disk_format_default(self, context):
+        """Gets an acceptable default image disk_format based on the schema.
+        """
+        # These preferred disk formats are in order:
+        # 1. we want qcow2 if possible (at least for backward compat)
+        # 2. vhd for xenapi and hyperv
+        # 3. vmdk for vmware
+        # 4. raw should be universally accepted
+        preferred_disk_formats = (
+            fields.DiskFormat.QCOW2,
+            fields.DiskFormat.VHD,
+            fields.DiskFormat.VMDK,
+            fields.DiskFormat.RAW,
+        )
+
+        # Get the image schema - note we don't cache this value since it could
+        # change under us. This looks a bit funky, but what it's basically
+        # doing is calling glanceclient.v2.Client.schemas.get('image').
+        image_schema = self._client.call(context, 2, 'get', 'image',
+                                         controller='schemas')
+        # get the disk_format schema property from the raw schema
+        disk_format_schema = (
+            image_schema.raw()['properties'].get('disk_format') if image_schema
+                                                                else {}
+        )
+        if disk_format_schema and 'enum' in disk_format_schema:
+            supported_disk_formats = disk_format_schema['enum']
+            # try a priority ordered list
+            for preferred_format in preferred_disk_formats:
+                if preferred_format in supported_disk_formats:
+                    return preferred_format
+            # alright, let's just return whatever is available
+            LOG.debug('Unable to find a preferred disk_format for image '
+                      'creation with the Image Service v2 API. Using: %s',
+                      supported_disk_formats[0])
+            return supported_disk_formats[0]
+
+        LOG.warning(_LW('Unable to determine disk_format schema from the '
+                        'Image Service v2 API. Defaulting to '
+                        '%(preferred_disk_format)s.'),
+                    {'preferred_disk_format': preferred_disk_formats[0]})
+        return preferred_disk_formats[0]
+
+    def _create_v2(self, context, sent_service_image_meta, data=None,
+                   force_activate=False):
+        # Glance v1 allows image activation without setting disk and
+        # container formats, v2 doesn't. It leads to the dirtiest workaround
+        # where we have to hardcode this parameters.
+        if force_activate:
+            data = ''
+            if 'disk_format' not in sent_service_image_meta:
+                sent_service_image_meta['disk_format'] = (
+                    self._get_image_create_disk_format_default(context)
+                )
+            if 'container_format' not in sent_service_image_meta:
+                sent_service_image_meta['container_format'] = 'bare'
+
+        location = sent_service_image_meta.pop('location', None)
+        image = self._client.call(
+            context, 2, 'create', **sent_service_image_meta)
+        image_id = image['id']
+
+        # Sending image location in a separate request.
+        if location:
+            image = self._add_location(context, image_id, location)
+
+        # If we have some data we have to send it in separate request and
+        # update the image then.
+        if data is not None:
+            image = self._upload_data(context, image_id, data)
+
+        return image
+
+    def update(self, context, image_id, image_meta, data=None,
+               purge_props=True):
+        """Modify the given image with the new data."""
+        sent_service_image_meta = _translate_to_glance(image_meta)
+        # NOTE(bcwaldon): id is not an editable field, but it is likely to be
+        # passed in by calling code. Let's be nice and ignore it.
+        sent_service_image_meta.pop('id', None)
+        sent_service_image_meta['image_id'] = image_id
+
+        try:
+            if purge_props:
+                # In Glance v2 we have to explicitly set prop names
+                # we want to remove.
+                all_props = set(self.show(
+                        context, image_id)['properties'].keys())
+                props_to_update = set(
+                        image_meta.get('properties', {}).keys())
+                remove_props = list(all_props - props_to_update)
+                sent_service_image_meta['remove_props'] = remove_props
+
+            image = self._update_v2(context, sent_service_image_meta, data)
+        except Exception:
+            _reraise_translated_image_exception(image_id)
+
+        return _translate_from_glance(image)
+
+    def _update_v2(self, context, sent_service_image_meta, data=None):
+        location = sent_service_image_meta.pop('location', None)
+        image_id = sent_service_image_meta['image_id']
+        image = self._client.call(
+            context, 2, 'update', **sent_service_image_meta)
+
+        # Sending image location in a separate request.
+        if location:
+            image = self._add_location(context, image_id, location)
+
+        # If we have some data we have to send it in separate request and
+        # update the image then.
+        if data is not None:
+            image = self._upload_data(context, image_id, data)
+
+        return image
+
+    def delete(self, context, image_id):
+        """Delete the given image.
+
+        :raises: ImageNotFound if the image does not exist.
+        :raises: NotAuthorized if the user is not an owner.
+        :raises: ImageNotAuthorized if the user is not authorized.
+
+        """
+        try:
+            self._client.call(context, 2, 'delete', image_id)
+        except glanceclient.exc.NotFound:
+            raise exception.ImageNotFound(image_id=image_id)
+        except glanceclient.exc.HTTPForbidden:
+            raise exception.ImageNotAuthorized(image_id=image_id)
+        return True
+
+
 def _extract_query_params(params):
     _params = {}
     accepted_params = ('filters', 'marker', 'limit',
@@ -520,6 +793,47 @@ def _extract_query_params(params):
     return _params
 
 
+def _extract_query_params_v2(params):
+    _params = {}
+    accepted_params = ('filters', 'marker', 'limit',
+                       'page_size', 'sort_key', 'sort_dir')
+    for param in accepted_params:
+        if params.get(param):
+            _params[param] = params.get(param)
+
+    # ensure filters is a dict
+    _params.setdefault('filters', {})
+    # NOTE(vish): don't filter out private images
+    _params['filters'].setdefault('is_public', 'none')
+
+    # adopt filters to be accepted by glance v2 api
+    filters = _params['filters']
+    new_filters = {}
+
+    for filter_ in filters:
+        # remove 'property-' prefix from filters by custom properties
+        if filter_.startswith('property-'):
+            new_filters[filter_.lstrip('property-')] = filters[filter_]
+        elif filter_ == 'changes-since':
+            # convert old 'changes-since' into new 'updated_at' filter
+            updated_at = 'gte:' + filters['changes-since']
+            new_filters['updated_at'] = updated_at
+        elif filter_ == 'is_public':
+            # convert old 'is_public' flag into 'visibility' filter
+            # omit the filter if is_public is None
+            is_public = filters['is_public']
+            if is_public.lower() in ('true', '1'):
+                new_filters['visibility'] = 'public'
+            elif is_public.lower() in ('false', '0'):
+                new_filters['visibility'] = 'private'
+        else:
+            new_filters[filter_] = filters[filter_]
+
+    _params['filters'] = new_filters
+
+    return _params
+
+
 def _is_image_available(context, image):
     """Check image availability.
 
@@ -564,12 +878,56 @@ def _is_image_available(context, image):
 def _translate_to_glance(image_meta):
     image_meta = _convert_to_string(image_meta)
     image_meta = _remove_read_only(image_meta)
+    # TODO(mfedosin): Remove this check once we move to glance V2
+    # completely and enable convert to v2 every time.
+    if not CONF.glance.use_glance_v1:
+        # v2 requires several additional changes
+        image_meta = _convert_to_v2(image_meta)
     return image_meta
 
 
+def _convert_to_v2(image_meta):
+    output = {}
+    for name, value in six.iteritems(image_meta):
+        if name == 'properties':
+            for prop_name, prop_value in six.iteritems(value):
+                # if allow_additional_image_properties is disabled we can't
+                # define kernel_id and ramdisk_id as None, so we have to omit
+                # these properties if they are not set.
+                if prop_name in ('kernel_id', 'ramdisk_id') and \
+                                prop_value is not None and \
+                                prop_value.strip().lower() in ('none', ''):
+                    continue
+                # in glance only string and None property values are allowed,
+                # v1 client accepts any values and converts them to string,
+                # v2 doesn't - so we have to take care of it.
+                elif prop_value is None or isinstance(
+                        prop_value, six.string_types):
+                    output[prop_name] = prop_value
+                else:
+                    output[prop_name] = str(prop_value)
+
+        elif name in ('min_ram', 'min_disk'):
+            output[name] = int(value)
+        elif name == 'is_public':
+            output['visibility'] = 'public' if value else 'private'
+        elif name in ('size', 'deleted'):
+            continue
+        else:
+            output[name] = value
+    return output
+
+
 def _translate_from_glance(image, include_locations=False):
-    image_meta = _extract_attributes(image,
-                                     include_locations=include_locations)
+    # TODO(mfedosin): Remove this check once we move to glance V2
+    # completely.
+    if CONF.glance.use_glance_v1:
+        image_meta = _extract_attributes(
+            image, include_locations=include_locations)
+    else:
+        image_meta = _extract_attributes_v2(
+            image, include_locations=include_locations)
+
     image_meta = _convert_timestamps_to_datetimes(image_meta)
     image_meta = _convert_from_string(image_meta)
     return image_meta
@@ -619,6 +977,8 @@ def _convert_to_string(metadata):
 
 
 def _extract_attributes(image, include_locations=False):
+    # TODO(mfedosin): Remove this function once we move to glance V2
+    # completely.
     # NOTE(hdd): If a key is not found, base.Resource.__getattr__() may perform
     # a get(), resulting in a useless request back to glance. This list is
     # therefore sorted, with dependent attributes as the end
@@ -653,7 +1013,9 @@ def _extract_attributes(image, include_locations=False):
                 output[attr] = getattr(image, attr, None)
         # NOTE(mdorman): 'size' attribute must not be 'None', so use 0 instead
         elif attr == 'size':
-            output[attr] = getattr(image, attr) or 0
+            # NOTE(mriedem): A snapshot image may not have the size attribute
+            # set so default to 0.
+            output[attr] = getattr(image, attr, 0) or 0
         else:
             # NOTE(xarses): Anything that is caught with the default value
             # will result in an additional lookup to glance for said attr.
@@ -666,6 +1028,31 @@ def _extract_attributes(image, include_locations=False):
     return output
 
 
+def _extract_attributes_v2(image, include_locations=False):
+    include_locations_attrs = ['direct_url', 'locations']
+    omit_attrs = ['self', 'schema', 'protected', 'virtual_size', 'file',
+                  'tags']
+    raw_schema = image.schema
+    schema = schemas.Schema(raw_schema)
+    output = {'properties': {}, 'deleted': False, 'deleted_at': None,
+              'disk_format': None, 'container_format': None, 'name': None,
+              'checksum': None}
+    for name, value in six.iteritems(image):
+        if (name in omit_attrs
+                or name in include_locations_attrs and not include_locations):
+            continue
+        elif name == 'visibility':
+            output['is_public'] = value == 'public'
+        elif name == 'size' and value is None:
+            output['size'] = 0
+        elif schema.is_base_property(name):
+            output[name] = value
+        else:
+            output['properties'][name] = value
+
+    return output
+
+
 def _remove_read_only(image_meta):
     IMAGE_ATTRIBUTES = ['status', 'updated_at', 'created_at', 'deleted_at']
     output = copy.deepcopy(image_meta)
@@ -737,12 +1124,22 @@ def get_remote_image_service(context, image_href):
     except ValueError:
         raise exception.InvalidImageRef(image_href=image_href)
 
-    image_service = GlanceImageService(client=glance_client)
+    # TODO(sbiswas7): Remove this check once we move to glance V2
+    # completely.
+    if CONF.glance.use_glance_v1:
+        image_service = GlanceImageService(client=glance_client)
+    else:
+        image_service = GlanceImageServiceV2(client=glance_client)
     return image_service, image_id
 
 
 def get_default_image_service():
-    return GlanceImageService()
+    # TODO(sbiswas7): Remove this check once we move to glance V2
+    # completely.
+    if CONF.glance.use_glance_v1:
+        return GlanceImageService()
+    else:
+        return GlanceImageServiceV2()
 
 
 class UpdateGlanceImage(object):
