@@ -21,16 +21,12 @@ from oslo_utils import timeutils
 import routes
 import six
 from six.moves import range
-import webob
 import webob.dec
-import webob.request
 
 from nova.api import auth as api_auth
 from nova.api import openstack as openstack_api
 from nova.api.openstack import api_version_request as api_version
-from nova.api.openstack import auth
 from nova.api.openstack import compute
-from nova.api.openstack.compute.legacy_v2 import limits
 from nova.api.openstack.compute import versions
 from nova.api.openstack import urlmap
 from nova.api.openstack import wsgi as os_wsgi
@@ -40,7 +36,6 @@ from nova.compute import vm_states
 from nova import context
 from nova.db.sqlalchemy import models
 from nova import exception as exc
-import nova.netconf
 from nova import objects
 from nova.objects import base
 from nova import quota
@@ -55,6 +50,8 @@ QUOTAS = quota.QUOTAS
 
 
 FAKE_UUID = 'aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa'
+FAKE_PROJECT_ID = '6a6a9c9eee154e9cb8cec487b98d36ab'
+FAKE_USER_ID = '5fae60f5cf4642609ddd31f71748beac'
 FAKE_UUIDS = {}
 
 
@@ -63,47 +60,19 @@ def fake_wsgi(self, req):
     return self.application
 
 
-def wsgi_app(inner_app_v2=None, fake_auth_context=None,
-        use_no_auth=False, ext_mgr=None, init_only=None):
-    if not inner_app_v2:
-        inner_app_v2 = compute.APIRouter(ext_mgr, init_only)
+def wsgi_app_v21(fake_auth_context=None, init_only=None, v2_compatible=False):
 
-    if use_no_auth:
-        api_v2 = openstack_api.FaultWrapper(auth.NoAuthMiddleware(
-              limits.RateLimitingMiddleware(inner_app_v2)))
-    else:
-        if fake_auth_context is not None:
-            ctxt = fake_auth_context
-        else:
-            ctxt = context.RequestContext('fake', 'fake', auth_token=True)
-        api_v2 = openstack_api.FaultWrapper(api_auth.InjectContext(ctxt,
-              limits.RateLimitingMiddleware(inner_app_v2)))
-
-    mapper = urlmap.URLMap()
-    mapper['/v2'] = api_v2
-    mapper['/'] = openstack_api.FaultWrapper(versions.Versions())
-    return mapper
-
-
-def wsgi_app_v21(inner_app_v21=None, fake_auth_context=None,
-        use_no_auth=False, ext_mgr=None, init_only=None, v2_compatible=False):
-    if not inner_app_v21:
-        inner_app_v21 = compute.APIRouterV21(init_only)
+    inner_app_v21 = compute.APIRouterV21()
 
     if v2_compatible:
         inner_app_v21 = openstack_api.LegacyV2CompatibleWrapper(inner_app_v21)
 
-    if use_no_auth:
-        api_v21 = openstack_api.FaultWrapper(auth.NoAuthMiddleware(
-              limits.RateLimitingMiddleware(inner_app_v21)))
+    if fake_auth_context is not None:
+        ctxt = fake_auth_context
     else:
-        if fake_auth_context is not None:
-            ctxt = fake_auth_context
-        else:
-            ctxt = context.RequestContext('fake', 'fake', auth_token=True)
-        api_v21 = openstack_api.FaultWrapper(api_auth.InjectContext(ctxt,
-              limits.RateLimitingMiddleware(inner_app_v21)))
-
+        ctxt = context.RequestContext('fake', 'fake', auth_token=True)
+    api_v21 = openstack_api.FaultWrapper(
+          api_auth.InjectContext(ctxt, inner_app_v21))
     mapper = urlmap.URLMap()
     mapper['/v2'] = api_v21
     mapper['/v2.1'] = api_v21
@@ -111,7 +80,7 @@ def wsgi_app_v21(inner_app_v21=None, fake_auth_context=None,
     return mapper
 
 
-def stub_out_key_pair_funcs(stubs, have_key_pair=True, **kwargs):
+def stub_out_key_pair_funcs(testcase, have_key_pair=True, **kwargs):
     def key_pair(context, user_id):
         return [dict(test_keypair.fake_keypair,
                      name='key', public_key='public_key', **kwargs)]
@@ -127,20 +96,10 @@ def stub_out_key_pair_funcs(stubs, have_key_pair=True, **kwargs):
         return []
 
     if have_key_pair:
-        stubs.Set(nova.db, 'key_pair_get_all_by_user', key_pair)
-        stubs.Set(nova.db, 'key_pair_get', one_key_pair)
+        testcase.stub_out('nova.db.key_pair_get_all_by_user', key_pair)
+        testcase.stub_out('nova.db.key_pair_get', one_key_pair)
     else:
-        stubs.Set(nova.db, 'key_pair_get_all_by_user', no_key_pair)
-
-
-def stub_out_rate_limiting(stubs):
-    def fake_rate_init(self, app):
-        super(limits.RateLimitingMiddleware, self).__init__(app)
-        self.application = app
-
-    v2_limits = nova.api.openstack.compute.legacy_v2.limits
-    stubs.Set(v2_limits.RateLimitingMiddleware, '__init__', fake_rate_init)
-    stubs.Set(v2_limits.RateLimitingMiddleware, '__call__', fake_wsgi)
+        testcase.stub_out('nova.db.key_pair_get_all_by_user', no_key_pair)
 
 
 def stub_out_instance_quota(test, allowed, quota, resource='instances'):
@@ -204,7 +163,7 @@ def stub_out_nw_api(test, cls=None, private=None, publics=None):
         publics = ['1.2.3.4']
 
     class Fake(object):
-        def __init__(self, skip_policy_check=False):
+        def __init__(self):
             pass
 
         def get_instance_nw_info(*args, **kwargs):
@@ -248,30 +207,24 @@ class FakeRequestContext(context.RequestContext):
 
 class HTTPRequest(os_wsgi.Request):
 
-    @staticmethod
-    def blank(*args, **kwargs):
-        kwargs['base_url'] = 'http://localhost/v2'
+    @classmethod
+    def blank(cls, *args, **kwargs):
+        defaults = {'base_url': 'http://localhost/v2'}
         use_admin_context = kwargs.pop('use_admin_context', False)
+        project_id = kwargs.pop('project_id', 'fake')
         version = kwargs.pop('version', os_wsgi.DEFAULT_API_VERSION)
-        out = os_wsgi.Request.blank(*args, **kwargs)
-        out.environ['nova.context'] = FakeRequestContext('fake_user', 'fake',
-                is_admin=use_admin_context)
+        defaults.update(kwargs)
+        out = super(HTTPRequest, cls).blank(*args, **defaults)
+        out.environ['nova.context'] = FakeRequestContext(
+            user_id='fake_user',
+            project_id=project_id,
+            is_admin=use_admin_context)
         out.api_version_request = api_version.APIVersionRequest(version)
         return out
 
 
-class HTTPRequestV21(os_wsgi.Request):
-
-    @staticmethod
-    def blank(*args, **kwargs):
-        kwargs['base_url'] = 'http://localhost/v2'
-        use_admin_context = kwargs.pop('use_admin_context', False)
-        version = kwargs.pop('version', os_wsgi.DEFAULT_API_VERSION)
-        out = os_wsgi.Request.blank(*args, **kwargs)
-        out.api_version_request = api_version.APIVersionRequest(version)
-        out.environ['nova.context'] = FakeRequestContext('fake_user', 'fake',
-                is_admin=use_admin_context)
-        return out
+class HTTPRequestV21(HTTPRequest):
+    pass
 
 
 class TestRouter(wsgi.Router):
@@ -355,13 +308,14 @@ def get_fake_uuid(token=0):
 
 def fake_instance_get(**kwargs):
     def _return_server(context, uuid, columns_to_join=None, use_slave=False):
+        if 'project_id' not in kwargs:
+            kwargs['project_id'] = 'fake'
         return stub_instance(1, **kwargs)
     return _return_server
 
 
 def fake_compute_get(**kwargs):
-    def _return_server_obj(context, uuid, want_objects=False,
-                           expected_attrs=None):
+    def _return_server_obj(context, uuid, expected_attrs=None):
         return stub_instance_obj(context, **kwargs)
     return _return_server_obj
 
@@ -411,8 +365,7 @@ def fake_instance_get_all_by_filters(num_servers=5, **kwargs):
 
 def fake_compute_get_all(num_servers=5, **kwargs):
     def _return_servers_objs(context, search_opts=None, limit=None,
-                             marker=None, want_objects=False,
-                             expected_attrs=None, sort_keys=None,
+                             marker=None, expected_attrs=None, sort_keys=None,
                              sort_dirs=None):
         db_insts = fake_instance_get_all_by_filters()(None,
                                                       limit=limit,
@@ -503,7 +456,7 @@ def stub_instance(id=1, user_id=None, project_id=None, host=None,
         "key_name": key_name,
         "key_data": key_data,
         "config_drive": config_drive,
-        "vm_state": vm_state or vm_states.BUILDING,
+        "vm_state": vm_state or vm_states.ACTIVE,
         "task_state": task_state,
         "power_state": power_state,
         "memory_mb": memory_mb,
@@ -664,7 +617,8 @@ def stub_snapshot_create(self, context, volume_id, name, description):
 
 
 def stub_compute_volume_snapshot_create(self, context, volume_id, create_info):
-    return {'snapshot': {'id': 100, 'volumeId': volume_id}}
+    return {'snapshot': {'id': "421752a6-acf6-4b2d-bc7a-119f9148cd8c",
+                         'volumeId': volume_id}}
 
 
 def stub_snapshot_delete(self, context, snapshot_id):
