@@ -103,7 +103,7 @@ class CellsSchedulerTestCase(test.TestCase):
                           'name': 'instance-00000001',
                           'hostname': 'meow',
                           'display_name': 'moo',
-                          'image_ref': 'fake_image_ref',
+                          'image_ref': uuidsentinel.fake_image_ref,
                           'user_id': self.ctxt.user_id,
                           # Test these as lists
                           'metadata': {'moo': 'cow'},
@@ -112,12 +112,14 @@ class CellsSchedulerTestCase(test.TestCase):
                           'project_id': self.ctxt.project_id}
 
         call_info = {'uuids': []}
-        block_device_mapping = [
+        block_device_mapping = objects.BlockDeviceMappingList(
+            objects=[
                 objects.BlockDeviceMapping(context=self.ctxt,
                     **fake_block_device.FakeDbBlockDeviceDict(
-                            block_device.create_image_bdm('fake_image_ref'),
-                            anon=True))
-               ]
+                            block_device.create_image_bdm(
+                                uuidsentinel.fake_image_ref),
+                        anon=True))
+               ])
 
         def _fake_instance_update_at_top(_ctxt, instance):
             call_info['uuids'].append(instance['uuid'])
@@ -131,6 +133,9 @@ class CellsSchedulerTestCase(test.TestCase):
         self.assertEqual(instance_uuids, call_info['uuids'])
 
         for count, instance_uuid in enumerate(instance_uuids):
+            bdms = db.block_device_mapping_get_all_by_instance(self.ctxt,
+                                                               instance_uuid)
+            self.assertIsNotNone(bdms)
             instance = db.instance_get_by_uuid(self.ctxt, instance_uuid)
             meta = utils.instance_meta(instance)
             self.assertEqual('cow', meta['moo'])
@@ -139,7 +144,8 @@ class CellsSchedulerTestCase(test.TestCase):
             self.assertEqual('meow', instance['hostname'])
             self.assertEqual('moo-%d' % (count + 1),
                              instance['display_name'])
-            self.assertEqual('fake_image_ref', instance['image_ref'])
+            self.assertEqual(uuidsentinel.fake_image_ref,
+                             instance['image_ref'])
 
     @mock.patch('nova.objects.Instance.update')
     def test_create_instances_here_pops_problematic_properties(self,
@@ -155,13 +161,22 @@ class CellsSchedulerTestCase(test.TestCase):
             'pci_requests': 'no thanks',
             'ec2_ids': 'prime',
         }
+        block_device_mapping = [
+                objects.BlockDeviceMapping(context=self.ctxt,
+                    **fake_block_device.FakeDbBlockDeviceDict(
+                            block_device.create_image_bdm(
+                                uuidsentinel.fake_image_ref),
+                            anon=True))
+               ]
 
         @mock.patch.object(self.scheduler.compute_api,
                            'create_db_entry_for_new_instance')
-        def test(mock_create_db):
+        @mock.patch.object(self.scheduler.compute_api,
+                           '_bdm_validate_set_size_and_instance')
+        def test(mock_bdm_validate, mock_create_db):
             self.scheduler._create_instances_here(
                 self.ctxt, [uuidsentinel.instance], values,
-                objects.Flavor(), 'foo', [], [])
+                objects.Flavor(), 'foo', [], block_device_mapping)
 
         test()
 
@@ -353,20 +368,11 @@ class CellsSchedulerTestCase(test.TestCase):
 
     def test_filter_schedule_skipping(self):
         # if a filter handles scheduling, short circuit
-
-        def _grab(filter_properties):
-            return None
-
-        self.stubs.Set(self.scheduler, '_grab_target_cells', _grab)
-
-        def _test(self, *args):
-            raise test.TestingException("shouldn't be called")
-
-        try:
-            self.scheduler._schedule_build_to_cells(None, None, None, _test,
-                                                    None)
-        except test.TestingException:
-            self.fail("Scheduling did not properly short circuit")
+        mock_func = mock.Mock()
+        self.scheduler._grab_target_cells = mock.Mock(return_value=None)
+        self.scheduler._schedule_build_to_cells(None, None, None,
+                                                mock_func, None)
+        mock_func.assert_not_called()
 
     def test_cells_filter_args_correct(self):
         # Re-init our fakes with some filters.
