@@ -22,7 +22,6 @@ import collections
 import copy
 import functools
 
-from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import excutils
 from oslo_utils import units
@@ -32,6 +31,7 @@ from oslo_vmware import pbm
 from oslo_vmware import vim_util as vutil
 import six
 
+import nova.conf
 from nova import exception
 from nova.i18n import _, _LE, _LI, _LW
 from nova.network import model as network_model
@@ -40,23 +40,7 @@ from nova.virt.vmwareapi import vim_util
 
 LOG = logging.getLogger(__name__)
 
-vmware_utils_opts = [
-    cfg.IntOpt('console_delay_seconds',
-               help='Set this value if affected by an increased network '
-                    'latency causing repeated characters when typing in '
-                    'a remote console.'),
-    cfg.StrOpt('serial_port_service_uri',
-               help='Identifies the remote system that serial port traffic '
-                    'will be sent to. If this is not set, no serial ports '
-                    'will be added to the created VMs.'),
-    cfg.StrOpt('serial_port_proxy_uri',
-               help='Identifies a proxy service that provides network access '
-                    'to the serial_port_service_uri. This option is ignored '
-                    'if serial_port_service_uri is not specified.'),
-    ]
-
-CONF = cfg.CONF
-CONF.register_opts(vmware_utils_opts, 'vmware')
+CONF = nova.conf.CONF
 
 ALL_SUPPORTED_NETWORK_DEVICES = ['VirtualE1000', 'VirtualE1000e',
                                  'VirtualPCNet32', 'VirtualSriovEthernetCard',
@@ -180,7 +164,7 @@ def _get_allocation_info(client_factory, limits, allocation_type):
     if limits.limit:
         allocation.limit = limits.limit
     else:
-        # Set as 'umlimited'
+        # Set as 'unlimited'
         allocation.limit = -1
     if limits.reservation:
         allocation.reservation = limits.reservation
@@ -277,6 +261,12 @@ def get_vm_create_spec(client_factory, instance, data_store_name,
     opt.value = instance.uuid
     extra_config.append(opt)
 
+    # enable to provide info needed by udev to generate /dev/disk/by-id
+    opt = client_factory.create('ns0:OptionValue')
+    opt.key = "disk.EnableUUID"
+    opt.value = True
+    extra_config.append(opt)
+
     port_index = 0
     for vif_info in vif_infos:
         if vif_info['iface_id']:
@@ -453,6 +443,8 @@ def _create_vif_spec(client_factory, vif_info, vif_limits=None):
                     'ns0:DistributedVirtualSwitchPortConnection')
         portgroup.switchUuid = network_ref['dvsw']
         portgroup.portgroupKey = network_ref['dvpg']
+        if 'dvs_port_key' in network_ref:
+            portgroup.portKey = network_ref['dvs_port_key']
         backing.port = portgroup
     else:
         backing = client_factory.create(
@@ -489,10 +481,11 @@ def _create_vif_spec(client_factory, vif_info, vif_limits=None):
     return network_spec
 
 
-def get_network_attach_config_spec(client_factory, vif_info, index):
+def get_network_attach_config_spec(client_factory, vif_info, index,
+                                   vif_limits=None):
     """Builds the vif attach config spec."""
     config_spec = client_factory.create('ns0:VirtualMachineConfigSpec')
-    vif_spec = _create_vif_spec(client_factory, vif_info)
+    vif_spec = _create_vif_spec(client_factory, vif_info, vif_limits)
     config_spec.deviceChange = [vif_spec]
     if vif_info['iface_id'] is not None:
         config_spec.extraConfig = [_iface_id_option_value(client_factory,
@@ -1280,7 +1273,10 @@ def get_all_cluster_mors(session):
                                         "ClusterComputeResource", ["name"])
         session._call_method(vutil, 'cancel_retrieval',
                              results)
-        return results.objects
+        if results.objects is None:
+            return []
+        else:
+            return results.objects
 
     except Exception as excep:
         LOG.warning(_LW("Failed to get cluster references %s"), excep)
